# coding=utf-8
"""
 :Description:    应用类
 :author          80071482/bony
 :@version         V1.1
 :@Date            2017年05月
"""


class App(object):
    def __init__(self, package):
        self.PACKAGE = package
